@@ -75,13 +75,13 @@ class RPM:
         if '.tar' not in self.source.suffixes[-2:]:
             raise ValueError('Expected tarball as source file')
 
-        self.tempdir = tempfile.mkdtemp()
+        self.tempdir = pathlib.Path(tempfile.mkdtemp())
         with tarfile.open(self.source) as tf:
             for name in tf.getnames():
                 if any(fnmatch.fnmatch(name, pat) for pat in ['*/pyproject.toml', '*/PKG-INFO', '*/setup.cfg']):
                     tf.extract(name, self.tempdir)
         try:
-            self.root = next(pathlib.Path(self.tempdir).glob('*/PKG-INFO')).parent
+            self.root = next(self.tempdir.glob('*/PKG-INFO')).parent
         except StopIteration:
             raise ValueError('Malformed source file: can’t find metadata')
         return self
@@ -95,7 +95,7 @@ class RPM:
         with tarfile.open(self.source) as tf:
             # Only extract files that will resolve within the destination directory − also skip existing files
             tempdir = self.tempdir.resolve()
-            safe_members = [name for name, dest in ((name, (tempdir / name).resolve()) for name in tf.getnames())
+            safe_members = [info for info, dest in ((info, (tempdir / info.name).resolve()) for info in tf.getmembers())
                             if dest.parts[:len(tempdir.parts)] == tempdir.parts and not dest.exists()]
             tf.extractall(self.tempdir, safe_members)
 
@@ -241,7 +241,7 @@ class RPM:
         metadata = {
             'build-requires': self.build_system['requires'],
             'build-backend': self.build_system['build-backend'],
-            'long-description': dist_meta.get_payload(),
+            'long-description': dist_meta.get_payload().replace('%', '%%'),
         }
 
         # See https://packaging.python.org/specifications/core-metadata/
@@ -252,9 +252,9 @@ class RPM:
             if key == 'Content-Type':
                 continue
             elif key in multiple_use:
-                metadata.setdefault(key.lower(), []).append(value)
+                metadata.setdefault(key.lower(), []).append(value.replace('%', '%%'))
             else:
-                metadata[key.lower()] = value
+                metadata[key.lower()] = value.replace('%', '%%')
 
         return {
             **metadata,
@@ -413,7 +413,14 @@ class RPM:
 
         # Determine the binary and source rpm names that should be built out of this spec file
         query = [*r'rpm -q --qf %{arch}/%{name}-%{version}-%{release}.%{arch}.rpm\n --specfile'.split(), str(spec_file)]
-        query_output = subprocess.run(query, capture_output=True, encoding='utf-8', check=True).stdout
+        try:
+            query_output = subprocess.run(query, capture_output=True, encoding='utf-8', check=True).stdout
+        except subprocess.CalledProcessError as err:
+            print(f'ERROR The command returned {err.returncode}:', ' '.join(
+                arg if ' ' not in arg else f"'{arg}'" if "'" not in arg else f'"{arg}"' for arg in query
+            ), file=sys.stderr)
+            print(err.stderr, file=sys.stderr)
+            raise RPMBuildError('rpm querying of specfile failed') from err
         binary_rpms = [pathlib.Path(out) for out in query_output.strip().split('\n')]
         source_rpm = pathlib.Path(binary_rpms[0].stem).with_suffix('.src.rpm')
 
